#pylint:disable=W0622
import os
from subprocess import Popen
import sys
import pathlib
import dill
from rich.console import Console
import time
import subprocess
from moyanlib import jsons
from platformdirs import PlatformDirs

dirs = PlatformDirs("PMPT", "MoYan")
IndexList = []
console = Console()

def getVer(baseVar):
    baseVar = baseVar + '.' + os.environ.get('GITHUB_RUN_ID', str(int(time.time()))[:6])
    return baseVar
    
__version__ = getVer('1.0.1')
def init():
    os.makedirs(dirs.user_data_dir,exist_ok=True)
    os.makedirs(os.path.join(dirs.user_data_dir,'Index'),exist_ok=True)
    os.makedirs(dirs.user_config_dir,exist_ok=True)
    os.makedirs(dirs.user_cache_dir,exist_ok=True)
    
def bubbleSort(arr):
    for i in range(1,len(arr)):
        for j in range(0, len(arr)-i):
            if arr[j]['priority'] < arr[j+1]['priority']:
                arr[j], arr[j + 1] = arr[j + 1], arr[j]
    return arr


def loadIndex():
    '''
    加载索引
    '''
    sourceList = jsons.load(open(os.path.join(dirs.user_config_dir,'Source.json')))
    sourceList = bubbleSort(sourceList)
    for source in sourceList:
        if not os.path.exists(os.path.join(dirs.user_data_dir,'Index',source['id']+'.pidx')):
            console.print(f'⚠️ [yellow]{source["url"]} did not create an index.[/yellow]')
            continue
        IndexFile = dill.load(open(os.path.join(dirs.user_data_dir,'Index',source['id']+'.pidx'),'rb')) # 加载索引
        IndexList.append(IndexFile)
    if len(IndexList) == 0:
        raise FileNotFoundError('No index. Run "pmpt update" first to update the index')

def runpip(command,other=None,dbg=False) -> Popen:
    '''
    运行pip
    '''
    if not other:
        other = []
    baseCommand = [sys.executable,'-m','pip']
    baseCommand.append(command)
    
    Command = baseCommand + other
    if dbg:
        console.print('Command to be run:',' '.join(Command))
    
    runClass = Popen(Command,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    for line in iter(runClass.stdout.readline, b''):
        # 在这里你可以对每一行输出进行处理
        line = line.decode('utf-8').strip()  # 将字节转换为字符串并去除换行符
        console.print(line) 
    runClass.communicate()
    return runClass

    



import uuid
from moyanlib import jsons
import os
from rich.table import Table
from . import util

def line_search(li, val):
    n = 0
    for i in li:
        print(i)
        if i['id'] == val:
            return i,n
        n += 1
    return None
    
def add(url,priority):
    sourceList = jsons.load(open(os.path.join(util.dirs.user_config_dir,'Source.json')))
    if url in sourceList:
        util.console.print('❌ [red]The source already exists[/red]')
        exit(1)
    sourceList.append({
        'url':url,
        'id':str(uuid.uuid4())[:8],
        'priority':priority
        
    })
    jsons.dump(sourceList,open(os.path.join(util.dirs.user_config_dir,'Source.json'),'w'))
  
    
def lists():
    sourceList = jsons.load(open(os.path.join(util.dirs.user_config_dir,'Source.json')))
    table = Table(show_header=True)
    table.add_column("ID",width=6)
    table.add_column('Priority',width=8)
    table.add_column('URL',width=25)
    
    for i in sourceList:
        table.add_row(
            i['id'],
            str(i['priority']),
            i['url']
        )
    util.console.print(table)
    
def remove(ids,yes):
    if not ids:
        lists()
        ids = util.console.input('Please enter the source sequence number to be deleted:')
        
    sourceList:list = jsons.load(open(os.path.join(util.dirs.user_config_dir,'Source.json')))
    source,n = line_search(sourceList,ids)

    if not source:
        util.console.print('❌ [red]The source for this ID does not exist[/red]')
    
    while True: # 是否允许安装
        if yes:
            break
               
        ye = util.console.input('Are you sure you want to delete? [Y/n]: ')
            
        if ye.lower() == 'y':
            break
        elif ye.lower() == 'n':
            util.console.print('🛑 [red]The user exited the program[/red]')
            exit(1)
        else:
            continue
            
    sourceList.remove(source)
    jsons.dump(sourceList,open(os.path.join(util.dirs.user_config_dir,'Source.json'),'w'))
  
def modify(ids,key,val):
    sourceList:list = jsons.load(open(os.path.join(util.dirs.user_config_dir,'Source.json')))
    source,n = line_search(sourceList,ids)
    if not source:
        util.console.print('❌ [red]The source for this ID does not exist[/red]')
    if key not in ['url','priority']:
        util.console.print('❌ [red]The item is not allowed to be modified or does not exist.[/red]')
    sourceList[n][key] = val
    jsons.dump(sourceList,open(os.path.join(util.dirs.user_config_dir,'Source.json'),'w'))